//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using MindFusion.Diagramming.Lanes;

using HatchBrush = MindFusion.Drawing.HatchBrush;
using Pen = MindFusion.Drawing.Pen;
using SolidBrush = MindFusion.Drawing.SolidBrush;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Lanes
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			// Initialize flowchart
			diagramView.SmoothingMode = SmoothingMode.AntiAlias;
			diagram.LinkShape = LinkShape.Cascading;
			diagram.LinkSegments = 2;
			diagram.LinkCascadeOrientation =
				Orientation.Horizontal;
			diagram.SnapToAnchor = SnapToAnchor.OnCreateOrModify;
			diagram.DiagramLinkStyle.Brush = new SolidBrush(Color.Black);
			diagram.LinkHeadShape = ArrowHeads.Triangle;
			diagram.LinkHeadShapeSize = 1.4f;
			diagram.ShadowOffsetX = 0.6f;
			diagram.ShadowOffsetY = 0.6f;
			diagram.DefaultShape = Shapes.Rectangle;
			diagram.AllowSelfLoops = false;

			pattern = new AnchorPattern(
				new AnchorPoint[]
					{
						new AnchorPoint(100, 50, false, true, MarkStyle.None),
						new AnchorPoint(0, 0, true, false, MarkStyle.None)
					});

			// Initialize grid
			Header header;
			Grid grid = diagram.LaneGrid;
			HeaderCollection columns = grid.ColumnHeaders;

			grid.MinHeaderSize = 8;
			grid.HookHeaders = false;
			grid.HeadersOnTop = false;
			grid.ColumnHeadersHeights = new float[] { 8, 8 };
			grid.AlignCells = false;
			grid.AllowResizeHeaders = true;

			for (int i = 1; i <= 8; i++)
			{
				header = new Header("Week " + i + ", 2017");
				header.SubHeaders.Add(new Header("S"));
				header.SubHeaders.Add(new Header("M"));
				header.SubHeaders.Add(new Header("T"));
				header.SubHeaders.Add(new Header("W"));
				header.SubHeaders.Add(new Header("T"));
				header.SubHeaders.Add(new Header("F"));
				header.SubHeaders.Add(new Header("S"));
				columns.Add(header);
			}

			for (int i = 1; i <= 6; i++)
				grid.RowHeaders.Add(new Header("Task " + i));

			int count = 8 * 7;
			for (int i = 0; i < count; i++)
			{
				Header column = grid.GetColumn(i);

				if (i % 7 != 0)
				{
					grid[column, null].Style.LeftBorderPen = null;
				}
				else
				{
					grid[column, null].Style.LeftBorderPen =
						new Pen(
							new HatchBrush(HatchStyle.Percent50, Color.Gray, Color.Transparent), 0);
				}

				if (i != count - 1)
					grid[column, null].Style.RightBorderPen = null;

				if (i % 7 == 0 ||
					i % 7 == 6)
				{
					grid[column, null].Style.BackgroundBrush =
                        new SolidBrush(Color.FromArgb(156, 170, 198));
				}
			}

			count = 6;
			for (int i = 0; i < count; i++)
			{
				Header row = grid.GetRow(i);

				if (i == 0)
				{
					grid[null, row].Style.TopBorderPen = null;
				}
				else
				{
					grid[null, row].Style.TopBorderPen =
						new Pen(
							new HatchBrush(HatchStyle.Percent50, Color.Gray, Color.Transparent), 0);
				}

				if (i != count - 1)
					grid[null, row].Style.BottomBorderPen = null;
			}

			diagram.EnableLanes = true;

			// Ensure the document is big enough to contain the grid
			float width = Math.Max(50, grid.ColumnCount * 6 + 20);
			float height = Math.Max(50, grid.RowCount * 6 + 20);

			diagram.Bounds = new RectangleF(0, 0, width, height);
			diagram.AlignToGrid = false;
		}

		/// <summary>
		/// Gets the size of a pixel measured in document units.
		/// </summary>
		private float GetPixel()
		{
			PointF pt1 = diagramView.ClientToDoc(new Point(0, 0));
			PointF pt2 = diagramView.ClientToDoc(new Point(1, 1));

			return pt2.X - pt1.X;
		}

		/// <summary>
		/// Fixes the segments of the specified arrow accordin
		/// to the position of its related boxes.
		/// </summary>
		private void FixArrow(DiagramLink a)
		{
			RectangleF originBounds = a.Origin.Bounds;
			RectangleF destBounds = a.Destination.Bounds;

			if (originBounds.Right < destBounds.Left)
			{
				a.SegmentCount = 2;
			}
			else
			{
				a.SegmentCount = 4;

				PointF pt0 = a.ControlPoints[0];
				PointF pt1;
				PointF pt2;
				PointF pt3 = a.ControlPoints[3];
				PointF pt4 = a.ControlPoints[4];

				pt1 = pt0;
				pt1.X += 2;

				pt2 = pt1;
				pt2.Y += 4;

				pt3.X = pt4.X;
				pt3.Y = pt2.Y;

				a.ControlPoints[1] = pt1;
				a.ControlPoints[2] = pt2;
				a.ControlPoints[3] = pt3;

				a.UpdateFromPoints();
			}
		}

		private void diagram_HitTestAdjustmentHandles(object sender, HitTestEventArgs e)
		{
			if (!(e.Item is ShapeNode))
			{
				e.HitResult = null;
				return;
			}

			ShapeNode box = e.Item as ShapeNode;

			if (!box.Bounds.Contains(e.MousePosition))
			{
				e.HitResult = null;
				return;
			}

			RectangleF left = box.Bounds;
			left.Width = 2;

			if (left.Contains(e.MousePosition))
			{
				e.HitResult = new NodeAdjustmentHandle(NodeHandleType.ResizeMiddleLeft);
				return;
			}

			RectangleF right = box.Bounds;
			right.X = right.Right - 2;
			right.Width = 2;

			if (right.Contains(e.MousePosition))
			{
				e.HitResult = new NodeAdjustmentHandle(NodeHandleType.ResizeMiddleRight);
				return;
			}

			RectangleF middle = box.Bounds;
			middle.X = middle.X + middle.Width / 4;
			middle.Width = middle.Width / 2;

			if (middle.Contains(e.MousePosition))
			{
				e.HitResult = NodeAdjustmentHandle.Move;
				return;
			}

			e.HitResult = null;
		}

		private void diagram_NodeCreated(object sender, NodeEventArgs e)
		{
			e.Node.HandlesStyle = HandlesStyle.Custom;
			e.Node.AnchorPattern = pattern;

			// Place the box within the grid
			RectangleF bounds = e.Node.Bounds;
			PointF topLeft = new PointF(bounds.Left, bounds.Top);

			Grid grid = diagram.LaneGrid;

			RectangleF cellBounds = RectangleF.Empty;
			grid.GetCellFromPoint(topLeft, ref cellBounds);

			float pixel = GetPixel();

			bounds.Y = cellBounds.Y + 4 * pixel;
			bounds.Height = cellBounds.Height - 10 * pixel;
			e.Node.Bounds = bounds;
		}

		private void diagram_NodeCreating(object sender, NodeValidationEventArgs e)
		{
			RectangleF bounds = e.Node.Bounds;
			PointF topLeft = new PointF(bounds.Left, bounds.Top);

			Grid grid = diagram.LaneGrid;

			RectangleF cellBounds = RectangleF.Empty;
			ICell cell = grid.GetCellFromPoint(topLeft, ref cellBounds);

			if (cell == null)
				e.Cancel = true;
		}

		private void diagram_SelectionChanged(object sender, EventArgs e)
		{
			foreach (ShapeNode node in diagram.Nodes)
			{
				node.Pen = new Pen(Color.Black, 0);
			}

			foreach (DiagramLink arrow in diagram.Links)
			{
				arrow.Pen = new Pen(Color.Black, 0);
				arrow.HeadPen = new Pen(Color.Black, 0);
				arrow.Brush = new SolidBrush(Color.Black);
			}

			foreach (ShapeNode node in diagram.Selection.Nodes)
			{
				node.Pen = new Pen(Color.Red, 0);
			}

			foreach (DiagramLink arrow in diagram.Selection.Links)
			{
				arrow.Pen = new Pen(Color.Red, 0);
				arrow.HeadPen = new Pen(Color.Red, 0);
				arrow.Brush = new SolidBrush(Color.Red);
			}
		}

		private void diagram_NodeModifying(object sender, NodeValidationEventArgs e)
		{
			RectangleF bounds = e.Node.Bounds;
			PointF topLeft = new PointF(bounds.Left, bounds.Top + bounds.Height / 2);

			Grid grid = diagram.LaneGrid;

			RectangleF cellBounds = RectangleF.Empty;
			ICell cell = grid.GetCellFromPoint(topLeft, ref cellBounds);

			if (cell == null)
				e.Cancel = true;
		}

		private void diagram_LinkCreated(object sender, LinkEventArgs e)
		{
			e.Link.AllowMoveStart = false;
			e.Link.AllowMoveEnd = false;
			e.Link.HandlesStyle = HandlesStyle.Custom;

			FixArrow(e.Link);
		}

		private void diagram_NodeModified(object sender, NodeEventArgs e)
		{
			// Place the box within the grid
			RectangleF bounds = e.Node.Bounds;
			PointF topLeft = new PointF(bounds.Left, bounds.Top + bounds.Height / 2);

			Grid grid = diagram.LaneGrid;

			RectangleF cellBounds = RectangleF.Empty;
			grid.GetCellFromPoint(topLeft, ref cellBounds);

			float pixel = GetPixel();

			bounds.Y = cellBounds.Y + 4 * pixel;
			bounds.Height = cellBounds.Height - 10 * pixel;
			e.Node.Bounds = bounds;

			// Fix links
			ArrayList links = new ArrayList();
			foreach (DiagramLink link in e.Node.IncomingLinks)
				links.Add(link);
			foreach (DiagramLink link in e.Node.OutgoingLinks)
				links.Add(link);

			foreach (DiagramLink link in links)
				FixArrow(link);
		}


		private AnchorPattern pattern = null;
	}
}